/*
 * =====================================================================================
 *
 *       Filename:  mymodule.c
 *
 *    Description:  netfilter testing for input hooks
 *
 *        Version:  1.0
 *        Created:  08/10/2011 12:03:50 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Mihai Cristea (UvA), 
 *        Company:  
 *
 * =====================================================================================
 */

#include <linux/module.h>
#include <linux/kernel.h> /* Needed for KERN_INFO */
#include <linux/init.h>   /* Needed for the macros */
#include <linux/skbuff.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include "prettyprint.h"

struct sk_buff *sock_buff;
struct iphdr *ip_header;
struct udphdr *udp_header;
static struct nf_hook_ops nfho;

static unsigned int hook_func(unsigned int hooknum, struct sk_buff *skb, const struct net_device *in, const struct net_device *out, int (*okfn)(struct sk_buff *))
{
  sock_buff = skb; 
  if (!sock_buff) {
     return NF_ACCEPT;
  } else {
    ip_header = (struct iphdr *)skb_network_header(sock_buff);
    if (!ip_header) {
       return NF_ACCEPT;
    } else {
      if (ip_header->protocol == IPPROTO_ICMP) 
      {
        printk("Dump ICMP packet, size of: %d\n", skb->len);
        displaydata(skb->data, skb->len);
        printk("\n\n");
        return NF_ACCEPT;
      } else {
       return NF_ACCEPT;
      }
    }
  }

}

static int __init mymodule_init(void)
{
  printk ("Register NF-hook for IN\n");
  nfho.hook     = hook_func;
  nfho.hooknum  = NF_INET_PRE_ROUTING;
//  nfho.hooknum  = NF_INET_LOCAL_IN;
  nfho.pf       = PF_INET;
  nfho.priority = NF_IP_PRI_FIRST;
//  nfho.priority = NF_IP_PRI_LAST;
  nf_register_hook(&nfho);
          
  return 0;
}

static void __exit mymodule_exit(void)
{
  printk ("Unregister NF-hook.\n");
  nf_unregister_hook(&nfho);
  return;
}

module_init(mymodule_init);
module_exit(mymodule_exit);

MODULE_LICENSE("GPL");

